from .base import *

class node(object):
    """
    cytoscape network interface as shown in CyREST's swagger documentation for 'node'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/node'

    def create_attribute(self, column=None, listType=None, namespace=None, network=None, coltype=None, verbose=False):
        """
        Creates a new node column.

        :param column (string, optional): Unique name of column.
        :param listType (string, optional): Can be one of integer, long, double,
            or string. = ['integer', 'long', 'double', 'string', 'boolean']
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be
            specified by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param coltype (string, optional): Can be one of integer, long, double,
            string, or list. = ['integer', 'long', 'double', 'string', 'boolean', 'list']
        :param verbose: print more

        :returns: {"columnName": columnName}
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["column","listType","namespace","network","type"],[column,listType,namespace,network,coltype])
        response=api(url=self.__url+"/create attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get(self, network=None, node=None, verbose=False):
        """
        Returns the SUID of a node that matches the passed parameters. If
        multiple nodes are found, only one will be returned, and a warning will
        be printed.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param node (string, optional): Selects a node by name, or, if the
            parameter has the prefix suid: selects a node by SUID.
        :param verbose: print more

        :returns: [ SUIDs of nodes that match the passed parameters ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","node"],[network,node])
        response=api(url=self.__url+"/get", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_attribute(self, columnList=None, namespace=None, network=None, nodeList=None, verbose=False):
        """
        Returns the attributes for the nodes passed as parameters.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks
            also support the shared namespace. Custom namespaces may be specified
            by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more

        :returns: [ { "name": "Q9UQ35"}, { "name": "Q4G0J3" } ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList", "namespace", "network","nodeList"],[columnList, namespace, network, nodeList])
        response=api(url=self.__url+"/get attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get_properties(self, network=None, nodeList=None, propertyList=None, verbose=False):
        """
        Returns the visual properties for the nodes that match the passed parameters.

        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes by
            their selection state. The pattern COLUMN:VALUE sets this parameter to
            any rows that contain the specified column value; if the COLUMN prefix
            is not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to match
            multiple values.
        :param propertyList (string, optional): A list of property names separated by commas.
        :param verbose: print more


        :returns: [
                    {
                      "SUID": 627,
                      "visualProperties": [
                        {
                          "visualProperty": "NODE_VISIBLE",
                          "value": true
                        },
                        {
                          "visualProperty": "NODE_PAINT",
                          "value": "#FF6666"
                        }
                      ]
                    },
                    {
                      "SUID": 566,
                      "visualProperties": [
                        {
                          "visualProperty": "NODE_VISIBLE",
                          "value": true
                        },
                        {
                          "visualProperty": "NODE_PAINT",
                          "value": "#FF6666"
                        }
                      ]
                    } ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network", "nodeList", "propertyList"],[network, nodeList, propertyList])
        response=api(url=self.__url+"/get properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def list(self, network=None, nodeList=None, verbose=False):
        """
        Returns a list of the node SUIDs associated with the passed network parameter.

        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value
            can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes by
            their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more

        :returns: [ 283,  295, 311 ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network", "nodeList"],[network, nodeList])
        response=api(url=self.__url+"/list", PARAMS=PARAMS, method="POST", verbose=verbose)

        return response

    def list_attributes(self, namespace=None, network=None, verbose=False):
        """
        Returns a list of column names assocated with nodes.

        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks also
            support the shared namespace. Custom namespaces may be specified by Apps.
        :param network (string, optional): Specifies a network by name, or by SUID
            if the prefix SUID: is used. The keyword CURRENT, or a blank value can
            also be used to specify the current network.
        :param verbose: print more

        :returns: [ list of column names assocated with nodes ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["namespace", "network"],[namespace, network])
        response=api(url=self.__url+"/list attributes", PARAMS=PARAMS, method="POST", verbose=verbose)

        return response

    def list_properties(self, network=None, verbose=False):
        """
        Returns a list of visual properties available for nodes.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: [ list of visual properties available for nodes ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/list properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def rename(self, network=None, newName=None, node=None, verbose=False):
        """
        Sets the value of the name column for the passed node.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param newName (string, optional): New name of the node ,
        :param node (string, optional): Selects a node by name, or, if the
            parameter has the prefix suid:, selects a node by SUID.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network", "newName", "node"],[network, newName, node])
        response=api(url=self.__url+"/rename", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def select_from_file(self, afile=None, verbose=False):
        """
        Selects nodes in the current network based on node names provided by a file.

        :param afile (string, optional): Path to file containing list of nodes to select
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["file"],[afile])
        response=api(url=self.__url+"/select from file", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def set_attribute(self, columnList=None, namespace=None, network=None, nodeList=None, valueList=None, verbose=False):
        """
        Sets the value of a specified column for the passed node or set of nodes.

        :param columnList (string, optional): A list of column names, separated
            by commas.
        :param namespace (string, optional): Node, Edge, and Network objects
            support the default, local, and hidden namespaces. Root networks also
            support the shared namespace. Custom namespaces may be specified by Apps.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network. ,
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes by
            their selection state. The pattern COLUMN:VALUE sets this parameter to
            any rows that contain the specified column value; if the COLUMN prefix
            is not used, the NAME column is matched by default. A list of COLUMN:VALUE
            pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,... can be used to
            match multiple values.
        :param valueList (string, optional): A list of values, separated by
            commas. List values can be included using the format [value1,value2].
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["columnList", "namespace", "network", "nodeList", "valueList"],[columnList, namespace, network, nodeList, valueList])
        response=api(url=self.__url+"/set attribute", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def set_properties(self, network=None, nodeList=None, propertyList=None, valueList=None, verbose=False):
        """
        Sets the value of a specified property for the passed node or set of nodes.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param propertyList (string, optional): A list of property names
            separated by commas.
        :param valueList (string, optional): A list of values separated by commas.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network", "nodeList", "propertyList", "valueList"],[network, nodeList, propertyList, valueList])
        response=api(url=self.__url+"/set properties", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    


from .base import *

class enrichmentmap(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/enrichmentmap'
        self.___url=url


    def getModelData(self, network, verbose=None):
        """
        

        :param network: Network name or SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/enrichmentmap/model/'+str(network)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getExpressionDataForNetwork(self, network, verbose=None):
        """
        

        :param network: Network name or SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/enrichmentmap/expressions/'+str(network)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getExpressionDataForNode(self, network, node, verbose=None):
        """
        

        :param network: Network name or SUID
        :param node: Node SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/enrichmentmap/expressions/'+str(network)+'/'+str(node)+'', method="GET", verbose=verbose, parse_params=False)
        return response


import sys
import requests
import urllib.request as urllib2		
import json


from json import JSONDecodeError

HOST = 'localhost'
IP=HOST # temporary
PORT = 1234
VERSION = 'v1'
BASE_URL = 'http://' + HOST + ':' + str(PORT) + '/' + VERSION + '/'
HEADERS = {'Content-Type': 'application/json'}
VERBOSE=False


SUID_LIST = 'suid'

BASE_URL_NETWORK = BASE_URL + 'networks'

def set_param(parameters,values):
    PARAMS={}
    for p,v in zip(parameters,values):
        if v:
            PARAMS[p]=v
    return PARAMS

def check_network(cyrest_network,network=None,verbose=False):
    if network is None:
        try:
            network_name=cyrest_network.network_name
        except AttributeError:
            network_name='CURRENT'
    else:
        network_name=network
    if verbose:
        print("Working on '%s' network." %str(network_name) )
    return network_name

def checkresponse(r,verbose=False):
    status=r.status_code
    if 200 <= status < 300:
        if verbose:
            print("response status "+status)
            sys.stdout.flush()
        res=None
    else:
        print(r, r.status_code)
        sys.stdout.flush()
        res="error::"+str(r.status_code)
    return res

def handle_status_codes(x,verbose=False):
    if type(x) == str:
        if "error::" in x:
            res=int(x.lstrip("error::"))
        else:
            res=None
    else:
        res=None

    if res or verbose:
        if res:
            res=res
        else:
            res=x
    else:
        res=None

    if not res:
        if x:
            res=x

    return res


def api(namespace=None, command="", PARAMS={}, body=None, host=HOST, port=str(PORT), version=VERSION, method="POST", verbose=VERBOSE, url=None, parse_params=True):
    """
    General function for interacting with Cytoscape API.

    :param namespace: namespace where the request should be executed. eg. "string"
    :param commnand: command to execute. eg. "protein query"
    :param PARAMs: a dictionary with the parameters. Check your swagger normaly running on
    http://localhost:1234/v1/swaggerUI/swagger-ui/index.html?url=http://localhost:1234/v1/commands/swagger.json
    :param host: cytoscape host address, default=cytoscape_host
    :param port: cytoscape port, default=1234
    :param version: API version
    :param method: type of http call, ie. "POST" or "GET" or "HELP".
    :param verbose: print more information

    :returns: For "POST" the data in the content's response. For "GET" None.

    eg.
    cytoscape("string","pubmed query",{"pubmed":"p53 p21","limit":"50"})
    """

    if url:
        baseurl=url
    else:
        if namespace:
            baseurl="http://"+str(host)+":"+str(port)+"/"+str(version)+"/commands/"+str(namespace)+"/"+str(command)
        else:
            baseurl="http://"+str(host)+":"+str(port)+"/"+str(version)+"/commands"

    if (method == "GET") or (method == "G"):
        URL=baseurl
        if verbose:
            print("'"+URL+"'")
            sys.stdout.flush()
        r = requests.get(url = URL, params=PARAMS)
        verbose_=checkresponse(r, verbose=verbose)
        if (verbose) or (verbose_):
            print("'"+URL+"'")
            sys.stdout.flush()

        if verbose_:
            res=verbose_
        else:
            res=r

    if (method == "DELETE"):
        URL=baseurl
        if verbose:
            print("'"+URL+"'")
            sys.stdout.flush()
        r = requests.delete(url = URL)
        verbose_=checkresponse(r, verbose=verbose)
        if (verbose) or (verbose_):
            print("'"+URL+"'")
            sys.stdout.flush()
        res=r.json()
        if len(res["errors"]) > 0:
            raise ValueError(res["errors"][0])

    elif (method == "POST") or (method == "P"):
        if verbose:
            print("'"+baseurl+"'")
            sys.stdout.flush()
        r = requests.post(url = baseurl, json = PARAMS)
        verbose_=checkresponse(r, verbose=verbose)
        if (verbose) or (verbose_):
            verbose=True
            print(r.content)
            sys.stdout.flush()

        res=r.json()
        if "errors" in res.keys():
            if len(res["errors"]) > 0:
                raise ValueError(res["errors"][0])
        if not verbose:
            if "data" in res.keys():
                res=res["data"]
        else:
            res=verbose_

    elif (method == "PUT"):
        if verbose:
            print("'"+baseurl+"'")
            sys.stdout.flush()
        r = requests.put(url = baseurl, json = body)

        verbose_=checkresponse(r, verbose=verbose)
        if (verbose) or (verbose_):
            verbose=True
            print(r.content)
            sys.stdout.flush()

        try:
            res=r.json()
            if "errors" in res.keys():
                if len(res["errors"]) > 0:
                    raise ValueError(res["errors"][0])
        except JSONDecodeError:
            if not r.text:
                res = {}
            else:
                raise

    elif (method=="HTML") or (method == "H") or (method=="HELP"):
        URL = baseurl
        if verbose:
            print("'"+URL+"'")
            sys.stdout.flush()
        response = requests.get(URL, params=PARAMS)

        res = response.text.split("\n")
        def clean(x):
            r=x.split("</p>")[0].split(">")[-1]
            return r
        res="\n".join([ clean(x) for x in res ])

    res=handle_status_codes(res,verbose=verbose)

    return res



from .base import *
import pandas as pd
import sys

class table(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'table'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/table'
        
    def add_row(self,keyValue=None,table=None,verbose=None):
        """
        Appends an additional row of empty cells to the current table.

        :param keyValue (string, optional): Specifies the primary key of a value in
                the row of a table Note that network, node, and edge tables must ha
            ve Long values as keys
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['keyValue','table'],[keyValue,table])
        response=api(url=self.__url+"/add row", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def create_column(self,columnName=None,listType=None,table=None,ntype=None,verbose=None):
        """
        Appends an additional column of attribute values to the current table.

        :param columnName (string, optional): The new column name
        :param listType (string, optional): Can be one of integer, long, double, or
                string.
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :param ntype (string, optional): Can be one of integer, long, double, string
            , or list.
        """
        PARAMS=set_param(['columnName','listType','table','type'],[columnName,\
        listType,table,ntype])
        response=api(url=self.__url+"/create column", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def create_table(self,keyColumn=None,keyColumnType=None,title=None,verbose=None):
        """
        Adds a new table to the network.

        :param keyColumn (string, optional): Specifies the name of a column in the
            table
        :param keyColumnType (string, optional): The syntactical type of the value
            used in the key
        :param title (string, optional): The name of the table used in the current
            network

        :returns: table SUID
        """
        PARAMS=set_param(['keyColumn','keyColumnType','title'],[keyColumn,\
        keyColumnType,title])
        response=api(url=self.__url+"/create table", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def delete_column(self,column=None,table=None,verbose=None):
        """
        Remove a column from a table, specified by its name. Returns the name of
        the column removed.

        :param column (string, optional): Specifies the name of a column in the tab
            le
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['column','table'],[column,table])
        response=api(url=self.__url+"/delete column", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def delete_row(self,keyValue=None,table=None,verbose=None):
        """
        Deletes a row from a table.Requires the table name or SUID and the row key.

        :param keyValue (string): Specifies the primary key of a value in the row o
            f a table
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['keyValue','table'],[keyValue,table])
        response=api(url=self.__url+"/delete row", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def destroy(self,table=None,verbose=None):
        """
        Removes the specified table from the network.

        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['table'],[table])
        response=api(url=self.__url+"/destroy", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def export(self,options=None,OutputFile=None,table=None,verbose=None):
        """
        Creates a file with name and writes the table there.

        :param options (string, optional): The format of the output file.
        :param OutputFile (string, optional): The path of the file to export the ta
            ble to. Note that the file will be overwritten if it exists.
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['options','OutputFile','table'],[options,OutputFile,table])
        response=api(url=self.__url+"/export", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def get_column(self,column=None,table=None,verbose=None):
        """
        Get the information about a table column.

        :param column (string, optional): Specifies the name of a column in the tab
            le
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :returns: information about a table column
        """
        PARAMS=set_param(['column','table'],[column,table])
        response=api(url=self.__url+"/get column", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def get_row(self,keyValue=None,table=None,verbose=None):
        """
        Returns the values in each column of a row of a table.

        :param keyValue (string, optional): Specifies a row of a table using the pr
            imary key as the indentifier
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :returns: values in each column of a row of a table

        """
        PARAMS = set_param(['keyValue','table'],[keyValue,table])
        response=api(url=self.__url+"/get row", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def get_value(self,column=None,keyValue=None,table=None,verbose=None):
        """
        Returns the value from a cell as specified by row and column ids.

        :param column (string, optional): Specifies the name of a column in the tab
            le
        :param keyValue (string, optional): Specifies a row of a table using the pr
            imary key as the indentifier
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :returns: value from a cell as specified by row and column ids

        """
        PARAMS=set_param(['column','keyValue','table'],[column,keyValue,table])
        response=api(url=self.__url+"/get value", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def import_file(self,caseSensitiveNetworkCollectionKeys=None,\
        caseSensitiveNetworkKeys=None,dataTypeList=None,\
        DataTypeTargetForNetworkCollection=None,DataTypeTargetForNetworkList=None,\
        delimiters=None,delimitersForDataList=None,afile=None,firstRowAsColumnNames=None,\
        KeyColumnForMapping=None,KeyColumnForMappingNetworkList=None,keyColumnIndex=None,\
        newTableName=None,startLoadRow=None,TargetNetworkCollection=None,\
        TargetNetworkList=None,WhereImportTable=None,verbose=None):
        """
        This uses a long list of input parameters to specify the attributes of
        the table, the mapping keys, and the destination table for the input.

        :param caseSensitiveNetworkCollectionKeys (string, optional): Determines wh
            ether capitalization is considered in matching and sorting
        :param caseSensitiveNetworkKeys (string, optional): Determines whether capi
            talization is considered in matching and sorting
        :param dataTypeList (string, optional): List of column data types ordered b
            y column index (e.g. "string,int,long,double,boolean,intlist" or jus
            t "s,i,l,d,b,il")
        :param DataTypeTargetForNetworkCollection (string, optional): Select whethe
            r to import the data as Node Table Columns, Edge Table Columns, or N
            etwork Table Columns
        :param DataTypeTargetForNetworkList (string, optional): The data type of th
            e targets
        :param delimiters (string, optional): The list of delimiters that separate
            columns in the table.
        :param delimitersForDataList (string, optional): The delimiters between ele
            ments of list columns in the table.
        :param afile (string): The path to the file that contains the table or netwo
            rk to be imported.
        :param firstRowAsColumnNames (string, optional): If the first imported row
            contains column names, set this to true.
        :param KeyColumnForMapping (string, optional): The column in the network to
                use as the merge key
        :param KeyColumnForMappingNetworkList (string, optional): The column in the
                network to use as the merge key
        :param keyColumnIndex (string, optional): The column that contains the key
            values for this import. These values will be used to match with the
            key values in the network.
        :param newTableName (string, optional): The title of the new table
        :param startLoadRow (string, optional): The first row of the input table to
                load. This allows the skipping of headers that are not part of the
            import.
        :param TargetNetworkCollection (string, optional): The network collection t
            o use for the table import
        :param TargetNetworkList (string, optional): The list of networks into whic
            h the table is imported
        :param WhereImportTable (string, optional): Determines what network(s) the
            imported table will be associated with (if any). A table can be impo
            rted into a Network Collection, Selected networks or to an unassigne
            d table.
        """
        PARAMS=set_param(['caseSensitiveNetworkCollectionKeys',\
        'caseSensitiveNetworkKeys','dataTypeList',\
        'DataTypeTargetForNetworkCollection','DataTypeTargetForNetworkList',\
        'delimiters','delimitersForDataList','file','firstRowAsColumnNames',\
        'KeyColumnForMapping','KeyColumnForMappingNetworkList','keyColumnIndex',\
        'newTableName','startLoadRow','TargetNetworkCollection','TargetNetworkList',\
        'WhereImportTable'],[caseSensitiveNetworkCollectionKeys,\
        caseSensitiveNetworkKeys,dataTypeList,DataTypeTargetForNetworkCollection,\
        DataTypeTargetForNetworkList,delimiters,delimitersForDataList,afile,\
        firstRowAsColumnNames,KeyColumnForMapping,KeyColumnForMappingNetworkList,\
        keyColumnIndex,newTableName,startLoadRow,TargetNetworkCollection,\
        TargetNetworkList,WhereImportTable])
        response=api(url=self.__url+"/import file", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def import_url(self,caseSensitiveNetworkCollectionKeys=None,\
        caseSensitiveNetworkKeys=None,dataTypeList=None,\
        DataTypeTargetForNetworkCollection=None,DataTypeTargetForNetworkList=None,\
        delimiters=None,delimitersForDataList=None,firstRowAsColumnNames=None,\
        KeyColumnForMapping=None,KeyColumnForMappingNetworkList=None,\
        keyColumnIndex=None,newTableName=None,startLoadRow=None,\
        TargetNetworkCollection=None,TargetNetworkList=None,url=None,\
        WhereImportTable=None,verbose=None):
        """
        Similar to Import Table this uses a long list of input parameters to
        specify the attributes of the table, the mapping keys, and the destination
        table for the input.

        :param caseSensitiveNetworkCollectionKeys (string, optional): Determines wh
            ether capitalization is considered in matching and sorting
        :param caseSensitiveNetworkKeys (string, optional): Determines whether capi
            talization is considered in matching and sorting
        :param dataTypeList (string, optional): List of column data types ordered b
            y column index (e.g. "string,int,long,double,boolean,intlist" or jus
            t "s,i,l,d,b,il")
        :param DataTypeTargetForNetworkCollection (string, optional): Select whethe
            r to import the data as Node Table Columns, Edge Table Columns, or N
            etwork Table Columns
        :param DataTypeTargetForNetworkList (string, optional): The data type of th
            e targets
        :param delimiters (string, optional): The list of delimiters that separate
            columns in the table.
        :param delimitersForDataList (string, optional): The delimiters between ele
            ments of list columns in the table.
        :param firstRowAsColumnNames (string, optional): If the first imported row
            contains column names, set this to true.
        :param KeyColumnForMapping (string, optional): The column in the network to
                use as the merge key
        :param KeyColumnForMappingNetworkList (string, optional): The column in the
                network to use as the merge key
        :param keyColumnIndex (string, optional): The column that contains the key
            values for this import. These values will be used to match with the
            key values in the network.
        :param newTableName (string, optional): The title of the new table
        :param startLoadRow (string, optional): The first row of the input table to
                load. This allows the skipping of headers that are not part of the
            import.
        :param TargetNetworkCollection (string, optional): The network collection t
            o use for the table import
        :param TargetNetworkList (string, optional): The list of networks into whic
            h the table is imported
        :param url (string): The URL of the file or resource that provides the tabl
            e or network to be imported.
        :param WhereImportTable (string, optional): Determines what network(s) the
            imported table will be associated with (if any). A table can be impo
            rted into a Network Collection, Selected networks or to an unassigne
            d table.
        """
        PARAMS=set_param(['caseSensitiveNetworkCollectionKeys',\
        'caseSensitiveNetworkKeys','dataTypeList','DataTypeTargetForNetworkCollection',\
        'DataTypeTargetForNetworkList','delimiters','delimitersForDataList',\
        'firstRowAsColumnNames','KeyColumnForMapping','KeyColumnForMappingNetworkList',\
        'keyColumnIndex','newTableName','startLoadRow','TargetNetworkCollection',\
        'TargetNetworkList','url','WhereImportTable'],[caseSensitiveNetworkCollectionKeys,\
        caseSensitiveNetworkKeys,dataTypeList,DataTypeTargetForNetworkCollection,\
        DataTypeTargetForNetworkList,delimiters,delimitersForDataList,\
        firstRowAsColumnNames,KeyColumnForMapping,KeyColumnForMappingNetworkList,\
        keyColumnIndex,newTableName,startLoadRow,TargetNetworkCollection,\
        TargetNetworkList,url,WhereImportTable])
        response=api(url=self.__url+"/import url", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def list_tables(self,includePrivate=None,namespace=None,atype=None,verbose=None):
        """
        Returns a list of the table SUIDs associated with the passed network parameter.

        :param includePrivate (string, optional): A boolean value determining wheth
            er to return private as well as public tables
        :param namespace (string, optional): An optional argument to contrain outpu
            t to a single namespace, or ALL
        :param atype (string, optional): One of ''network'', ''node'', ''edge'', ''u
            nattached'', ''all'', to constrain the type of table listed
        :returns: list of the table SUIDs associated with the passed network parameter.
        """
        PARAMS=set_param(['includePrivate','namespace','type'],\
        [includePrivate,namespace,atype])
        response=api(url=self.__url+"/list", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def list_columns(self,table=None,verbose=None):
        """
        Returns the list of columns in the table.

        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :returns: list of columns in the table.
        """
        PARAMS=set_param(['table'],[table])
        response=api(url=self.__url+"/list columns", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def list_rows(self,rowList=None,table=None,verbose=None):
        """
        Returns the list of primary keys for each of the rows in the specified table.

        :param rowList (string, optional): Specifies a list of rows. The pattern CO
            LUMN:VALUE sets this parameter to any rows that contain the specifie
            d column value; if the COLUMN prefix is not used, the NAME column is
                matched by default. A list of COLUMN:VALUE pairs of the format COLU
            MN1:VALUE1,COLUMN2:VALUE2,... can be used to match multiple values.
            This parameter can also be set to all to include all rows.
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['rowList','table'],[rowList,table])
        response=api(url=self.__url+"/list rows", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    
    def merge(self,DataTypeTargetForNetworkCollection=None,\
        dataTypeTargetForNetworkList=None,mergeType=None,SourceMergeColumns=None,\
        SourceMergeKey=None,SourceTable=None,TargetKeyNetworkCollection=None,\
        TargetMergeKey=None,TargetNetworkCollection=None,TargetNetworkList=None,\
        UnassignedTable=None,WhereMergeTable=None,verbose=None):
        """
        Merge tables together joining around a designated key column. Depending
        on the arguments, might merge into multiple local tables.

        :param DataTypeTargetForNetworkCollection (string, optional): The collectio
            n of networks where the merged table will reside
        :param dataTypeTargetForNetworkList (string, optional):
        :param mergeType (string, optional): A choice between ''Copy Columns'' and
            ''Link To Columns'' that determines if replicates are created
        :param SourceMergeColumns (string, optional): A list of columns that will b
            e brought into the merged table
        :param SourceMergeKey (string, optional): The name of the columns that exis
            ts in both tables and is used to correlate rows
        :param SourceTable (string, optional): The name of the table used as the ba
            se data in the merge
        :param TargetKeyNetworkCollection (string, optional): The name of the prima
            ry column about which the merge is made
        :param TargetMergeKey (string, optional):
        :param TargetNetworkCollection (string, optional): The group of networks th
            at will be merged into the source table
        :param TargetNetworkList (string, optional): The list of networks where the
                merged table will be added
        :param UnassignedTable (string, optional):
        :param WhereMergeTable (string, optional): The destination path of the resu
            ltant merged table. The choices are ''Network Collection'', ''Select
            ed Networks'', or ''All Unassigned Tables''.
        """
        PARAMS=set_param(['DataTypeTargetForNetworkCollection','dataTypeTargetForNetworkList','mergeType','SourceMergeColumns','SourceMergeKey','SourceTable','TargetKeyNetworkCollection','TargetMergeKey','TargetNetworkCollection','TargetNetworkList','UnassignedTable','WhereMergeTable'],\
        [DataTypeTargetForNetworkCollection,dataTypeTargetForNetworkList,mergeType,SourceMergeColumns,SourceMergeKey,SourceTable,TargetKeyNetworkCollection,TargetMergeKey,TargetNetworkCollection,TargetNetworkList,UnassignedTable,WhereMergeTable])
        response=api(url=self.__url+"/merge", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response



    def rename_column(self,columnName=None,newColumnName=None,table=None,verbose=None):
        """
        Changes the name of a specified column in the table.

        :param columnName (string): The name of the column that will be renamed.
        :param newColumnName (string): The new name of the column.
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        """
        PARAMS=set_param(['columnName','newColumnName','table'],[columnName,newColumnName,table])
        response=api(url=self.__url+"/rename column", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    
    def set_title(self,table=None,title=None,verbose=None):
        """
        Changes the visible identifier of a single table.

        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :param title (string, optional): The name of the table used in the current
            network
        """
        PARAMS=set_param(['table','title'],[table,title])
        response=api(url=self.__url+"/set title", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    
    def set_values(self,columnName=None,rowList=None,table=None,value=None,verbose=None):
        """
        Set all the values in the specified list of rows with a single value.

        :param columnName (string, optional): Specifies the name of a column in the
                table
        :param rowList (string, optional): Specifies a list of rows. The pattern CO
            LUMN:VALUE sets this parameter to any rows that contain the specifie
            d column value; if the COLUMN prefix is not used, the NAME column is
                matched by default. A list of COLUMN:VALUE pairs of the format COLU
            MN1:VALUE1,COLUMN2:VALUE2,... can be used to match multiple values.
            This parameter can also be set to all to include all rows.
        :param table (string, optional): Specifies a table by table name. If the pr
            efix SUID: is used, the table corresponding the SUID will be returne
            d.
        :param value (string, optional): The value to set the columns in the select
            ed rows to. This should be a string value, which will be converted t
            o the appropriate column type.
        """
        PARAMS=set_param(['columnName','rowList','table','value'],[columnName,rowList,table,value])
        response=api(url=self.__url+"/set values", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def getTable(self, columns=None, table=None, network = "current", namespace='default', verbose=VERBOSE):
        """
        Gets tables from cytoscape.

        :param table: table to retrieve eg. node
        :param columns: columns to retrieve in list format
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param namespace (string, optional): Node, Edge, and Network objects support
            the default, local, and hidden namespaces. Root networks also support the
            shared namespace. Custom namespaces may be specified by Apps.

        :returns: a pandas dataframe
        """

        u=self.__url 
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        if type(network) != int:
            network=check_network(self,network,verbose=verbose)
            PARAMS=set_param(["columnList","namespace","network"],["SUID",namespace,network])
            network=api(namespace="network", command="get attribute",PARAMS=PARAMS, host=host,port=str(port),version=version)
            network=network[0]["SUID"]

        df=pd.DataFrame()
        def target(column):
            URL="http://"+str(host)+":"+str(port)+"/v1/networks/"+str(network)+"/tables/"+namespace+table+"/columns/"+column
            if verbose:
                print("'"+URL+"'")
                sys.stdout.flush()
            response = urllib2.urlopen(URL)
            response = response.read()
            colA=json.loads(response)

            col=pd.DataFrame()    
            colHeader=colA["name"]
            colValues=colA["values"]
            col[colHeader]=colValues
            return col

        ncols=["name"]
        for c in columns:
            ncols.append(c.replace(" ","%20") )
        for c in ncols:
            try:
                col=target(c)
                df=pd.concat([df,col],axis=1)
            except:
                print("Could not find "+c)
                sys.stdout.flush()

        df.index=df["name"].tolist()
        df=df.drop(["name"],axis=1)
        return df

    def loadTableData(self, df, df_key='index',table="node", table_key_column = "name", \
        network="current",namespace="default",verbose=False):
        """
        Loads tables into cytoscape.

        :param df: a pandas dataframe to load
        :param df_key: key column in df, default="index"
        :param table: target table, default="node"
        :param table_key_column: table key column, default="name"
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param namespace (string, optional): Node, Edge, and Network objects support
            the default, local, and hidden namespaces. Root networks also support the
            shared namespace. Custom namespaces may be specified by Apps.
        :param verbose: print more information

        :returns: output of put request
        """

        u=self.__url 
        host=u.split("//")[1].split(":")[0]
        port=u.split(":")[2].split("/")[0]
        version=u.split(":")[2].split("/")[1]

        if type(network) != int:
            network=check_network(self,network,verbose=verbose)
            
            PARAMS=set_param(["columnList","namespace","network"],["SUID",namespace,network])
            networkID=api(namespace="network", command="get attribute",PARAMS=PARAMS, host=host,port=str(port),version=version)
            
            PARAMS=set_param(["columnList","namespace","network"],["name",namespace,network])
            networkname=api(namespace="network", command="get attribute",PARAMS=PARAMS, host=host,port=str(port),version=version)
                    
            network=networkID[0]["SUID"]
            networkname=networkname[0]["name"]

        tmp=df.copy()
        if df_key!="index":
            tmp.index=tmp[df_key].tolist()
            tmp=tmp.drop([df_key],axis=1)
                
        tablen=networkname+" default node"

        data=[]

        for c in tmp.columns.tolist():
            tmpcol=tmp[[c]].dropna()
            for r in tmpcol.index.tolist():
                cell={}
                cell[str(table_key_column)]=str(r) # {"name":"p53"}
                val=tmpcol.loc[r,c]
                if type(val) != str:
                    val=float(val)
                cell[str(c)]=val
                data.append(cell)


        upload={"key":table_key_column,"dataKey":table_key_column,\
                "data":data}


        URL="http://"+str(host)+":"+str(port)+"/v1/networks/"+str(network)+"/tables/"+namespace+table  
        if verbose:
            print("'"+URL+"'", upload)
            sys.stdout.flush()
        r = requests.put(url = URL, json = upload)
        if verbose:
            print(r)
        checkresponse(r)
        res=r.content
        return res

    def getTableCount(verbose=None):
        """
        Returns the number of global tables.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.url+'tables/count', method="GET", verbose=verbose, parse_params=False)
        return response



from .base import *

class networks(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/networks'
        self.___url=url


    def collapseGroup(self, networkId, groupNodeId, verbose=None):
        """
        Collapses the group specified by the `groupNodeId` and `networkId` parameters.

        :param networkId: SUID of the Network
        :param groupNodeId: SUID of the Node representing the Group
        :param verbose: print more

        :returns: 204: Group collapsed; 500: Failed to collapse group
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups/'+str(groupNodeId)+'/collapse', method="GET", verbose=verbose, parse_params=False)
        return response


    def expandGroup(self, networkId, groupNodeId, verbose=None):
        """
        Expands the group specified by the `groupNodeId` and `networkId` parameters.

        :param networkId: SUID of the Network
        :param groupNodeId: SUID of the Node representing the Group
        :param verbose: print more

        :returns: 204: Group expanded; 500: Failed to expand group
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups/'+str(groupNodeId)+'/expand', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNetworkViewCount(self, networkId, verbose=None):
        """
        Returns a count of the Network Views available for the Network specified by the `networkId` parameter.
        
        Cytoscape can have multiple views per network model, but this feature is not exposed in the Cytoscape GUI. GUI access is limited to the first available view only.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def getFirstImageAsPdf(self, networkId, h, verbose=None):
        """
        Returns a PDF of the first available Network View for the Network specified by the `networkId` parameter.
        
        Default size is 600 px

        :param networkId: SUID of the Network
        :param h: Height of the image. Width is set automatically -- Not required, can be None
        :param verbose: print more

        :returns: 200: PDF image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/first.pdf', PARAMS={'h':h}, method="GET", verbose=verbose, parse_params=False)
        return response


    def updateView(self, networkId, viewId, objectType, objectId, bypass, body, verbose=None):
        """
        Updates the Visual Properties in the object specified by the `objectId` and `objectType` parameters in the Network View specified by the `viewId` and `networkId` parameters.
        
        Examples of Visual Properties:
        
        ```
        {
        "visualProperty": "NODE_BORDER_WIDTH",
        "value": 2
        }
        ```
        
        ```
        {
        "visualProperty": "EDGE_TRANSPARENCY",
        "value": 170
        }```
        
        ```
        {
        "visualProperty": "NETWORK_BACKGROUND_PAINT",
        "value": "#000000"
        }```
        
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)
        
        Note that this sets the Visual Properties temporarily unless the `bypass` parameter is set to `true`. If the `bypass` parameter is set to `true`, the Visual Style will be overridden by these Visual Property values. If the `bypass` parameter is not used or is set to `false`, any Visual Properties set will return to those defined in the Visual Style if the Network View is updated.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param objectId: SUID of the Object
        :param bypass: Bypass the Visual Style with these Visual Properties -- Not required, can be None
        :param body: A list of Visual Properties and their values.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'', method="PUT", body=body, verbose=verbose)
        return response


    def getView(self, networkId, viewId, objectType, objectId, verbose=None):
        """
        Gets a list of Visual Properties for the Object specified by the `objectId` and `objectType` parameters in the Network View specified by the `viewId` and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param objectId: SUID of the Object
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getTableAsCsv(self, networkId, tableType, verbose=None):
        """
        Returns a CSV representation of the table specified by the `networkId` and `tableType` parameters. All column names are included in the first row.

        :param networkId: SUID of the network containing the table
        :param tableType: Table type
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'.csv', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNeighbours(self, networkId, nodeId, verbose=None):
        """
        Returns the neighbors of the node specified by the `nodeId` and `networkId` parameters as a list of SUIDs.

        :param networkId: SUID of the network containing the node.
        :param nodeId: SUID of the node
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/'+str(nodeId)+'/neighbors', method="GET", verbose=verbose, parse_params=False)
        return response


    def putNetworkVisualPropBypass(self, networkId, viewId, visualProperty, body, verbose=None):
        """
        Bypasses the Visual Style of the Network with the Visual Property specificed by the `visualProperty`, `viewId`, and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param visualProperty: Name of the Visual Property
        :param body: A Visual Property and its value.
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network/'+str(visualProperty)+'/bypass', method="PUT", body=body, verbose=verbose)
        return response


    def deleteNetworkVisualProp(self, networkId, viewId, visualProperty, verbose=None):
        """
        Deletes the bypass Visual Property specificed by the `visualProperty`, `viewId`, and `networkId` parameters. When this is done, the Visual Property will be defined by the Visual Style
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network/'+str(visualProperty)+'/bypass', method="DELETE", verbose=verbose)
        return response


    def getNetworkVisualPropBypass(self, networkId, viewId, visualProperty, verbose=None):
        """
        Gets the bypass Visual Property specified by the `visualProperty`, `viewId`, and `networkId` parameters.  The response is the Visual Property that is used in place of the definition provided by the Visual Style.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network/'+str(visualProperty)+'/bypass', method="GET", verbose=verbose, parse_params=False)
        return response


    def setCurrentNetwork(self, body, verbose=None):
        """
        Sets the current network.

        :param body: SUID of the Network -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/currentNetwork', method="PUT", body=body, verbose=verbose)
        return response


    def getCurrentNetwork(self, verbose=None):
        """
        Returns the current network.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/currentNetwork', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNetworkVisualProp(self, networkId, viewId, visualProperty, verbose=None):
        """
        Gets the Network Visual Property specificed by the `visualProperty`, `viewId`, and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network/'+str(visualProperty)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def createNetworkFromSelected(self, networkId, title, verbose=None):
        """
        Creates new sub-network from current selection, with the name specified by the `title` parameter.
        
        Returns the SUID of the new sub-network.

        :param networkId: SUID of the network containing the selected nodes and edges
        :param title: Name for the new sub-network -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        PARAMS=set_param(['networkId','title'],[networkId,title])
        response=api(url=self.___url+'networks/'+str(networkId)+'', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteNetwork(self, networkId, verbose=None):
        """
        Deletes the network specified by the `networkId` parameter.

        :param networkId: SUID of the network to delete
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'', method="DELETE", verbose=verbose)
        return response


    def getNetwork(self, networkId, verbose=None):
        """
        Returns the Network specified by the `networkId` parameter with all associated tables in [Cytoscape.js](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-js-json) format

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'', method="H", verbose=verbose, parse_params=False)
        return response


    def updateTable(self, networkId, tableType, body, class_, verbose=None):
        """
        Updates the table specified by the `tableType` and `networkId` parameters.  New columns will be created if they do not exist in the target table.
        
        Current limitations:
        * Numbers are handled as Double
        * List column is not supported in this version

        :param networkId: SUID containing the table
        :param tableType: Type of table
        :param body: The data with which to update the table.
        :param class_: None -- Not required, can be None
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'', method="PUT", body=body, verbose=verbose)
        return response


    def getTable(self, networkId, tableType, verbose=None):
        """
        Returns the table specified by the `networkId` and 'tableType' parameters.

        :param networkId: SUID of the network containing the table
        :param tableType: Table type
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'', method="H", verbose=verbose, parse_params=False)
        return response


    def getSingleVisualPropertyValue(self, networkId, viewId, objectType, objectId, visualProperty, verbose=None):
        """
        Gets the Visual Property specificed by the `visualProperty` parameter for the node or edge specified by the `objectId` parameter in the Network View specified by the `viewId` and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param objectId: SUID of the Object
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'/'+str(visualProperty)+'', method="H", verbose=verbose, parse_params=False)
        return response


    def updateViews(self, networkId, viewId, objectType, bypass, body, verbose=None):
        """
        Updates multiple node or edge Visual Properties as defined by the `objectType` parameter, in the Network View specified by the `viewId` and `networkId` parameters.
        
        Examples of Visual Properties:
        
        ```
        {
        "visualProperty": "NODE_BORDER_WIDTH",
        "value": 2
        }
        ```
        ```
        {
        "visualProperty": "NODE_BORDER_PAINT",
        "value": "#CCCCCC"
        }
        ```
        
        ```
        {
        "visualProperty": "EDGE_TRANSPARENCY",
        "value": 170
        }```
        ```
        {
        "visualProperty": "EDGE_PAINT",
        "value": "#808080"
        }```
        
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)
        
        Note that this sets the Visual Properties temporarily unless the `bypass` parameter is set to `true`. If the `bypass` parameter is set to `true`, the Visual Style will be overridden by these Visual Property values. If the `bypass` parameter is not used or is set to `false`, any Visual Properties set will return to those defined in the Visual Style if the Network View is updated.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param bypass: Bypass the Visual Style with these Visual Properties -- Not required, can be None
        :param body: A list of Objects with Visual Properties.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'', method="PUT", body=body, verbose=verbose)
        return response


    def getViews(self, networkId, viewId, objectType, visualProperty, verbose=None):
        """
        Returns a list of all Visual Property values for the Visual Property specified by the `visualProperty` and `objectType` parameters, in the Network View specified by the `viewId` and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param visualProperty: Name of the Visual Property -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'', PARAMS={'visualProperty':visualProperty}, method="H", verbose=verbose, parse_params=False)
        return response


    def getTableAsTsv(self, networkId, tableType, verbose=None):
        """
        Returns a TSV (tab delimited text) representation of the table specified by the `networkId` and `tableType` parameters. All column names are included in the first row.

        :param networkId: SUID of the network containing the table
        :param tableType: Table type
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'.tsv', method="G", verbose=verbose, parse_params=False)
        return response


    def getNetworkViewAsCx(self, networkId, viewId, verbose=None):
        """
        Returns the Network View specified by the `viewId` and `networkId` parameters in [CX format](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-cx)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'.cx', method="G", verbose=verbose, parse_params=False)
        return response


    def getImageAsPdf(self, networkId, viewId, verbose=None):
        """
        Returns a PDF of the Network View specified by the `viewId` and `networkId` parameters.
        
        Default size is 500 px.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param verbose: print more

        :returns: 200: PDF image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'.pdf', method="G", verbose=verbose, parse_params=False)
        return response


    def getNetworkPointer(self, networkId, nodeId, verbose=None):
        """
        If the node specified by the `nodeId` and `networkId` parameters has an associated nested network, returns the SUID of the nested network.

        :param networkId: SUID of the network containing the node
        :param nodeId: SUID of the node
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/'+str(nodeId)+'/pointer', method="GET", verbose=verbose, parse_params=False)
        return response


    def getTables(self, networkId, verbose=None):
        """
        Returns every table in the network specified by the `networkId` parameter.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateColumnName(self, networkId, tableType, body, verbose=None):
        """
        Renames an existing column in the table specified by the `tableType` and `networkId` parameters.

        :param networkId: SUID of the network containing the table
        :param tableType: Table Type
        :param body: Old and new column name
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns', method="PUT", body=body, verbose=verbose)
        return response


    def createColumn(self, networkId, tableType, body, verbose=None):
        """
        Creates a new, empty column in the table specified by the `tableType` parameter, in the network specified by the `networkId` parameter.
        
        This resource can also accept an array of new columns to create multiple columns.

        :param networkId: SUID of the Network
        :param tableType: Table Type
        :param body: New Column Info
        :param verbose: print more

        :returns: 201: Column(s) createed; 412: Could not process column JSON
        """

        PARAMS=set_param(['networkId','tableType','body'],[networkId,tableType,body])
        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def getColumnNames(self, networkId, tableType, verbose=None):
        """
        Returns all the columns in the table specified by the `networkId` and `tableType` parameters.

        :param networkId: SUID of the network containing the table
        :param tableType: Table Type
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns', method="GET", verbose=verbose, parse_params=False)
        return response


    def getRow(self, networkId, tableType, primaryKey, verbose=None):
        """
        Gets a row matching the value specified by the `primaryKey` parameter from the table specified by the `tableType` and `networkId` parameters.
        
        Data is represented by column names and their values.
        
        ```json
        {
        "name": "Hodor 1",
        "value": 0.11,
        "matched": false
        ...
        }
        ```

        :param networkId: SUID of the network containing the table
        :param tableType: Table type
        :param primaryKey: Primary key of the row Object, normally an SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/rows/'+str(primaryKey)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getEdgeDirected(self, networkId, edgeId, verbose=None):
        """
        Returns true if the edge specified by the `edgeId` and `networkId` parameters is directed.

        :param networkId: SUID of the network containing the edge
        :param edgeId: SUID of the edge
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/'+str(edgeId)+'/isDirected', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNetworkCount(self, verbose=None):
        """
        Returns the number of networks in current Cytoscape session.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNetworkView(self, networkId, viewId, file, verbose=None):
        """
        Gets the Network View specified by the `viewId` and `networkId` parameters.
        
        If the `file` parameter is left unspecified, the response will contain data in [Cytoscape.js](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-js-json) format.
        
        If the `file` parameter is specified, the Network View will be written to a file, and the response will contain the location of the file in the following format:
        
        ```
        
        {
        "file": "/media/HD1/myFiles/networkView.sif"
        }
        ```
        
        The format of the output file is defined by the extension of the `file` parameter.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param file: A path to a file relative to the current directory. The format of the file written is defined by the file extension.

    | Extension   | Details    |
    | ----------- | -----------|
    | .cys        | Cytoscape Style format |
    | .xml/.xgmml | [XGMML](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html?highlight=xgmml#xgmml-format) format |
    | .nnf        | [NNF](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#nnf) format |
    | .sif        | [SIF](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#sif-format) format |
    | .cyjs       | [Cytoscape.js](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-js-json) format |
    -- Not required, can be None
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'', PARAMS={'file':file}, method="GET", verbose=verbose, parse_params=False)
        return response


    def putSingleVisualPropertyValueBypass(self, networkId, viewId, objectType, objectId, visualProperty, body, verbose=None):
        """
        Bypasses the Visual Style of the object specified by the `objectId` and `objectType` parameters, in the Network View specified by the `viewId` and `networkId` parameters. The Visual Property included in the message body will be used instead of the definition provided by the Visual Style.
        
        Examples of Visual Properties:
        
        ```
        {
        "visualProperty": "NODE_BORDER_WIDTH",
        "value": 2
        }
        ```
        
        ```
        {
        "visualProperty": "EDGE_TRANSPARENCY",
        "value": 170
        }```
        
        ```
        {
        "visualProperty": "NETWORK_BACKGROUND_PAINT",
        "value": "#000000"
        }```
        
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: Network SUID
        :param viewId: Network View SUID
        :param objectType: Type of Object
        :param objectId: SUID of the Object
        :param visualProperty: Name of the Visual Property
        :param body: A Visual Property and its value.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'/'+str(visualProperty)+'/bypass', method="PUT", body=body, verbose=verbose)
        return response


    def deleteSingleVisualPropertyValueBypass(self, networkId, viewId, objectType, objectId, visualProperty, verbose=None):
        """
        Deletes the bypass Visual Property specified by the `visualProperty` parameter from the object specified by the `objectId` and `objectType` parameters in the Network View Specified by the `viewId` and `networkId` parameters. When this is done, the Visual Property will be defined by the Visual Style
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param objectType: Type of Object
        :param objectId: SUID of Object
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'/'+str(visualProperty)+'/bypass', method="DELETE", verbose=verbose)
        return response


    def getSingleVisualPropertyValueBypass(self, networkId, viewId, objectType, objectId, visualProperty, verbose=None):
        """
        Gets the bypass Visual Property specified by the `visualProperty` parameter from the object specified by the `objectId` and `objectType` parameters in the Network View Specified by the `viewId` and `networkId` parameters. The response is the Visual Property that is used in place of the definition provided by the Visual Style.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: Network SUID
        :param viewId: Network View SUID
        :param objectType: Type of Object
        :param objectId: SUID of the Object
        :param visualProperty: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/'+str(objectType)+'/'+str(objectId)+'/'+str(visualProperty)+'/bypass', method="GET", verbose=verbose, parse_params=False)
        return response


    def deleteNode(self, networkId, nodeId, verbose=None):
        """
        Deletes the node specified by the `nodeId` and `networkId` parameters.

        :param networkId: SUID of the network containing the node.
        :param nodeId: SUID of the node
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/'+str(nodeId)+'', method="DELETE", verbose=verbose)
        return response


    def getNode(self, networkId, nodeId, verbose=None):
        """
        Returns a node with its associated row data.

        :param networkId: SUID of the network containing the node
        :param nodeId: SUID of the node
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/'+str(nodeId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def createEdge(self, networkId, body, verbose=None):
        """
        Add new edge(s) to the network.  Body should include an array of new node names.
        
        Returns and array of objects with fields itentifying the SUIDs of the new edges along with source and target SUIDs.

        :param networkId: SUID of the network to add edges to.
        :param body: Array of new edges
        :param verbose: print more

        :returns: 200: successful operation
        """

        PARAMS=set_param(['networkId','body'],[networkId,body])
        response=api(url=self.___url+'networks/'+str(networkId)+'/edges', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteAllEdges(self, networkId, verbose=None):
        """
        Delete all the edges from the network specified by the `networkId` parameter.

        :param networkId: SUID of the network to delete edges from
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges', method="DELETE", verbose=verbose)
        return response


    def getEdges(self, networkId, column, query, verbose=None):
        """
        Returns a list of all edges in the network specified by the `networkId` parameter as SUIDs.
        
        If the `column` and `query` parameters are specified, the results will be limited to rows in the edge table where the value in the column specified by the `column` parameter matches the value specified by the `query` parameter.

        :param networkId: SUID of the network containing the edges
        :param column: The name of the column that will be queried for matches. -- Not required, can be None
        :param query: The value to be matched. -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges', PARAMS={'column':column, 'query':query}, method="GET", verbose=verbose, parse_params=False)
        return response


    def setSelectedEdges(self, networkId, body, verbose=None):
        """
        Sets as selected the edges specified by the `suids` and `networkId` parameters.
        
        Returns a list of selected SUIDs.

        :param networkId: SUID of the network containing the edges
        :param body: Array of edge SUIDs to select -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/selected', method="PUT", body=body, verbose=verbose)
        return response


    def getSelectedEdges(self, networkId, verbose=None):
        """
        Gets the selected edges in the network specified by the `networkId` parameter. The results are presented as a list of SUIDs.

        :param networkId: SUID of the network containing the edges
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/selected', method="GET", verbose=verbose, parse_params=False)
        return response


    def getFirstImageAsPng(self, networkId, h, verbose=None):
        """
        Returns a PNG image of the first available Network View for the Network specified by the `networkId` parameter.
        
        Default size is 600 px

        :param networkId: SUID of the Network
        :param h: Height of the image. Width is set automatically -- Not required, can be None
        :param verbose: print more

        :returns: 200: PNG image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/first.png', PARAMS={'h':h}, method="GET", verbose=verbose, parse_params=False)
        return response


    def getCell(self, networkId, tableType, primaryKey, columnName, verbose=None):
        """
        Return the value of a cell specified by the `primaryKey` and `columnName` parameters in the table specified by the `tableType` and `networkId` parameters.
        
        Returns a JSON representation of a String, Boolean, Number, or List.

        :param networkId: SUID of the network containing the table
        :param tableType: Table type
        :param primaryKey: Primary key of the row Object, normally an SUID
        :param columnName: Name of the Column
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/rows/'+str(primaryKey)+'/'+str(columnName)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def setSelectedNodes(self, networkId, body, verbose=None):
        """
        Sets as selected the nodes specified by the `suids` and `networkId` parameters.
        
        Returns a list of selected SUIDs.

        :param networkId: SUID of the network containing the nodes
        :param body: Array of node SUIDs to select -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/selected', method="PUT", body=body, verbose=verbose)
        return response


    def getSelectedNodes(self, networkId, verbose=None):
        """
        Gets the selected nodes in the network specified by the `networkId` parameter. The results are presented as a list of SUIDs.

        :param networkId: SUID of the network containing the nodes
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/selected', method="GET", verbose=verbose, parse_params=False)
        return response


    def deleteGroup(self, networkId, groupNodeId, verbose=None):
        """
        Deletes the group specified by the `groupNodeId` and `networkId` parameters. The nodes and edges that the group contained will remain present in the network, however the node used to identify the Group will be deleted.

        :param networkId: SUID of the Network
        :param groupNodeId: SUID of the Node representing the Group
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups/'+str(groupNodeId)+'', method="DELETE", verbose=verbose)
        return response


    def getGroup(self, networkId, groupNodeId, verbose=None):
        """
        Returns the group specified by the `groupNodeId` and `networkId` parameters.

        :param networkId: SUID of the Network
        :param groupNodeId: SUID of the Node representing the Group
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups/'+str(groupNodeId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getEdgeCount(self, networkId, verbose=None):
        """
        Returns the number of edges in the network specified by the `networkId` parameter.

        :param networkId: SUID of the network containing the edges
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateColumnValues(self, networkId, tableType, columnName, default, body, verbose=None):
        """
        Sets the values for cells in the table specified by the `tableType` and `networkId` parameters.
        
        If the 'default` parameter is not specified, the message body should consist of key-value pairs with which to set values.
        
        If the `default` parameter is specified, its value will be used for every cell in the column. This is useful to set columns like "selected."

        :param networkId: SUID of the network containing the table
        :param tableType: The type of table
        :param columnName: Name of the column in which to set values
        :param default: Default Value. If this value is provided, all cells will be set to this. -- Not required, can be None
        :param body: Array of SUID Keyed values
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns/'+str(columnName)+'', method="PUT", body=body, verbose=verbose)
        return response


    def deleteColumn(self, networkId, tableType, columnName, verbose=None):
        """
        Deletes the column specified by the `columnName` parameter from the table speficied by the `tableType` and `networkId` parameters.

        :param networkId: SUID of the network containing the table from which to delete the column
        :param tableType: Table Type from which to delete the column
        :param columnName: Name of the column to delete
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns/'+str(columnName)+'', method="DELETE", verbose=verbose)
        return response


    def getColumnValues(self, networkId, tableType, columnName, verbose=None):
        """
        Returns all the values for the column specified by the `columnType` parameter, in the table specified by the `networkId` and `tableType` parameters.

        :param networkId: SUID of the Network
        :param tableType: Type of Table
        :param columnName: Name of the Column
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/columns/'+str(columnName)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def createNode(self, networkId, body, verbose=None):
        """
        Adds new nodes to the network specified by the `networkId` parameter. The `name` column will be populated by the contents of the message body.

        :param networkId: SUID of the network containing the node.
        :param body: Array of new node names
        :param verbose: print more

        :returns: 201: ; 412: 
        """

        PARAMS=set_param(['networkId','body'],[networkId,body])
        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteAllNodes(self, networkId, verbose=None):
        """
        Delete all the nodes from the network specified by the `networkId` parameter.

        :param networkId: SUID of the network to delete nodes from
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes', method="DELETE", verbose=verbose)
        return response


    def getNodes(self, networkId, column, query, verbose=None):
        """
        Returns a list of all nodes in the network specified by the `networkId` parameter as SUIDs.
        
        If the `column` and `query` parameters are specified, the results will be limited to rows in the node table where the value in the column specified by the `column` parameter matches the value specified by the `query` parameter.

        :param networkId: SUID of the network containing the nodes
        :param column: The name of the column that will be queried for matches. -- Not required, can be None
        :param query: The value to be matched. -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes', PARAMS={'column':column, 'query':query}, method="GET", verbose=verbose, parse_params=False)
        return response


    def setCurrentNetworkView(self, body, verbose=None):
        """
        Sets the current Network View.

        :param body: SUID of the Network View -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/views/currentNetworkView', method="PUT", body=body, verbose=verbose)
        return response


    def getCurrentNetworkView(self, verbose=None):
        """
        Returns the current Network View.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/views/currentNetworkView', method="GET", verbose=verbose, parse_params=False)
        return response


    def getGroupCount(self, networkId, verbose=None):
        """
        Returns the number of groups in the network

        :param networkId: Network SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def deleteEdge(self, networkId, edgeId, verbose=None):
        """
        Deletes the edge specified by the `edgeId` and `networkId` parameters.

        :param networkId: SUID of the network containing the edge.
        :param edgeId: SUID of the edge
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/'+str(edgeId)+'', method="DELETE", verbose=verbose)
        return response


    def getEdge(self, networkId, edgeId, verbose=None):
        """
        Returns an edge with its associated row data.

        :param networkId: SUID of the network containing the edge
        :param edgeId: SUID of the edge
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/'+str(edgeId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getRows(self, networkId, tableType, verbose=None):
        """
        Returns all rows from the table specified by `networkId` and `tableType` parameters. Returns a JSON representation of an array of rows.
        
        ```
        [
        {
            "SUID": 101,
            "gene_name": "brca1",
            "exp": 0.1
        },
        {
            "SUID": 102,
            "gene_name": "brca2",
            "exp": 0.2
        }
        ]
        ```

        :param networkId: SUID of the network containing the table
        :param tableType: Table Type
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/tables/'+str(tableType)+'/rows', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNodeCount(self, networkId, verbose=None):
        """
        Returns the number of nodes in the network specified by the `networkId` parameter.

        :param networkId: SUID of the network containing the nodes
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def getNeighborsSelected(self, networkId, verbose=None):
        """
        Returns the neighbors of the nodes currently selected in the network specified by the `networkId` parameter as a list of SUIDs.
        
        Note that this does not include the nodes in the original selection.

        :param networkId: SUID of the network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/selected/neighbors', method="GET", verbose=verbose, parse_params=False)
        return response


    def deleteFirstNetworkView(self, networkId, verbose=None):
        """
        Deletes the first available Network View for the Network specified by the `networkId` parameter. Cytoscape can have multiple views per network model, but this feature is not exposed in the Cytoscape GUI. GUI access is limited to the first available view only.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/first', method="DELETE", verbose=verbose)
        return response


    def getFirstNetworkView(self, networkId, file, verbose=None):
        """
        This returns the first view of the network. Cytoscape can have multiple views per network model, but this feature is not exposed in the Cytoscape GUI. GUI access is limited to the first available view only.
        
        If the `file` parameter is left unspecified, the response will contain data in [Cytoscape.js](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-js-json) format.
        
        If the `file` parameter is specified, the Network View will be written to a file, and the response will contain the location of the file in the following format:
        
        ```
        
        {
        "file": "/media/HD1/myFiles/networkView.sif"
        }
        ```
        
        The format of the output file is defined by the extension of the `file` parameter.

        :param networkId: SUID of the Network
        :param file: A path to a file relative to the current directory. The format of the file written is defined by the file extension.

    | Extension   | Details    |
    | ----------- | -----------|
    | .cys        | Cytoscape Style format |
    | .xml/.xgmml | [XGMML](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html?highlight=xgmml#xgmml-format) format |
    | .nnf        | [NNF](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#nnf) format |
    | .sif        | [SIF](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#sif-format) format |
    | .cyjs       | [Cytoscape.js](http://manual.cytoscape.org/en/stable/Supported_Network_File_Formats.html#cytoscape-js-json) format |
    -- Not required, can be None
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/first', PARAMS={'file':file}, method="GET", verbose=verbose, parse_params=False)
        return response


    def getAdjEdges(self, networkId, nodeId, verbose=None):
        """
        Returns a list of connected edges as SUIDs for the node specified by the `nodeId` and `networkId` parameters.

        :param networkId: SUID of the network containing the node
        :param nodeId: SUID of the node
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/nodes/'+str(nodeId)+'/adjEdges', method="GET", verbose=verbose, parse_params=False)
        return response


    def createNetworkView(self, networkId, verbose=None):
        """
        Creates a new Network View for the Network specified by the `networkId` parameter.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 201: Network View SUID
        """

        PARAMS=set_param(['networkId'],[networkId])
        response=api(url=self.___url+'networks/'+str(networkId)+'/views', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteAllNetworkViews(self, networkId, verbose=None):
        """
        Deletes all Network Views available in the Network specified by the `networkId` parameter. Cytoscape can have multiple views per network model, but this feature is not exposed in the Cytoscape GUI. GUI access is limited to the first available view only.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views', method="DELETE", verbose=verbose)
        return response


    def getAllNetworkViews(self, networkId, verbose=None):
        """
        Returns an array of all network views belonging to the network specified by the `networkId` paramter. The response is a list of Network SUIDs.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: An array of Network View SUIDs
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views', method="GET", verbose=verbose, parse_params=False)
        return response


    def getEdgeComponent(self, networkId, edgeId, type, verbose=None):
        """
        Returns the SUID of the source or target node of the edge specified by the `edgeId` and `networkId` parameters.
        
        Return values can be in one of two formats, depending on the value specified in the `type` parameter:
        
        ```
        {
        "source": 101
        }
        ```
        
        ```
        {
        "target": 102
        }
        ```

        :param networkId: SUID of the network containing the edge
        :param edgeId: SUID of the edge
        :param type: The node type to return
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/edges/'+str(edgeId)+'/'+str(type)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateNetworkView(self, networkId, viewId, bypass, body, verbose=None):
        """
        Updates the Visual Properties in the Network View specified by the `viewId` and `networkId` parameters.
        
        Example Visual Properties:
        ```
        {
        "visualProperty": "NETWORK_BACKGROUND_PAINT",
        "value": "#000000"
        }```
        ```
        {
        "visualProperty": "NETWORK_CENTER_X_LOCATION",
        "value": 250
        }```
        
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)
        
        Note that this sets the Visual Properties temporarily unless the `bypass` parameter is set to `true`. If the `bypass` parameter is set to `true`, the Visual Style will be overridden by these Visual Property values. If the `bypass` parameter is not used or is set to `false`, any Visual Properties set will return to those defined in the Visual Style if the Network View is updated.

        :param networkId: Network SUID
        :param viewId: Network View SUID
        :param bypass: Bypass the Visual Style with these properties -- Not required, can be None
        :param body: A list of Visual Properties and their values.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network', method="PUT", body=body, verbose=verbose)
        return response


    def getNetworkVisualProps(self, networkId, viewId, verbose=None):
        """
        Returns a list of the Visual Properties for the Network View specified by the `viewId` and `networkId` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'/network', method="GET", verbose=verbose, parse_params=False)
        return response


    def createGroup(self, networkId, body, verbose=None):
        """
        Create a new group in the network specified by the parameter `networkId`. The contents are specified the message body.

        :param networkId: SUID of the Network
        :param body: New Group name and contents
        :param verbose: print more

        :returns: 200: successful operation
        """

        PARAMS=set_param(['networkId','body'],[networkId,body])
        response=api(url=self.___url+'networks/'+str(networkId)+'/groups', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteAllGroups(self, networkId, verbose=None):
        """
        Deletes all groups in the network specified by `networkId` parameter. The nodes and edges that the groups contained will remain present in the network, however the nodes used to identify the Groups will be deleted.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups', method="DELETE", verbose=verbose)
        return response


    def getAllGroups(self, networkId, verbose=None):
        """
        Returns a list of all the groups in the network specified by the `networkId` parameter.

        :param networkId: Network SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/groups', method="GET", verbose=verbose, parse_params=False)
        return response


    def getImageAsSvg(self, networkId, viewId, h, verbose=None):
        """
        Returns an SVG image of the Network View specified by the `viewId` and `networkId` parameters.
        
        Default size is 600 px.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param h: Height of the image. Width is set automatically -- Not required, can be None
        :param verbose: print more

        :returns: 200: SVG image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'.svg', PARAMS={'h':h}, method="GET", verbose=verbose, parse_params=False)
        return response


    def getImageAsPng(self, networkId, viewId, h, verbose=None):
        """
        Returns a PNG image of the Network View specified by the `viewId` and `networkId` parameters.
        
        Default size is 600 px.

        :param networkId: SUID of the Network
        :param viewId: SUID of the Network View
        :param h: Height of the image. Width is set automatically -- Not required, can be None
        :param verbose: print more

        :returns: 200: PNG image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/'+str(viewId)+'.png', PARAMS={'h':h}, method="GET", verbose=verbose, parse_params=False)
        return response


    def getFirstImageAsSvg(self, networkId, h, verbose=None):
        """
        Returns an SVG image of the first available Network View for the Network specified by the `networkId` parameter.
        
        Default size is 600 px

        :param networkId: SUID of the Network
        :param h: Height of the image. Width is set automatically -- Not required, can be None
        :param verbose: print more

        :returns: 200: SVG image stream.
        """

        response=api(url=self.___url+'networks/'+str(networkId)+'/views/first.svg', PARAMS={'h':h}, method="GET", verbose=verbose, parse_params=False)
        return response




import pandas as pd

from . import cytoscapejs as cyjs


def from_dataframe(df,
                   source_col='source',
                   target_col='target',
                   interaction_col='interaction',
                   name='From DataFrame',
                   edge_attr_cols=[]):
    """
    Utility to convert Pandas DataFrame object into Cytoscape.js JSON

    :param df: Dataframe to convert.
    :param source_col: Name of source column.
    :param target_col: Name of target column.
    :param interaction_col: Name of interaction column.
    :param name: Name of network.
    :param edge_attr_cols: List containing other columns to consider in df as
        edges' attributes.
    :return: Dictionary version of df.
    """

    network = cyjs.get_empty_network(name=name)
    nodes = set()
    if edge_attr_cols is None:
        edge_attr_cols = []

    for index, row in df.iterrows():
        s = row[source_col]
        t = row[target_col]
        if s not in nodes:
            nodes.add(s)
            source = get_node(s)
            network['elements']['nodes'].append(source)
        if t not in nodes:
            nodes.add(t)
            target = get_node(t)
            network['elements']['nodes'].append(target)

        extra_values = {column: row[column]
                        for column in edge_attr_cols
                        if column in df.columns}

        network['elements']['edges'].append(
            get_edge(s, t, interaction=row[interaction_col], **extra_values)
        )
    return network


def to_dataframe(network,
                 interaction='interaction',
                 default_interaction='-',
                 edges_attr_cols=[]):
    """
    Utility to convert a Cytoscape dictionary into a Pandas Dataframe.

    :param network: Dictionary to convert.
    :param interaction: Name of interaction column.
    :param default_interaction: Default value for missing interactions.
    :param edges_attr_cols: List containing other edges' attributes to include
        in the Dataframe, if present in network.
    :return: Converted Pandas Dataframe.
    """

    edges = network['elements']['edges']
    if edges_attr_cols is None:
        edges_attr_cols = []
    edges_attr_cols = sorted(edges_attr_cols)

    network_array = []
    # the set avoids duplicates
    valid_extra_cols = set()
    for edge in edges:
        edge_data = edge['data']
        source = edge_data['source']
        target = edge_data['target']
        if interaction in edge_data:
            itr = edge_data[interaction]
        else:
            itr = default_interaction
        extra_values = []
        for extra_column in edges_attr_cols:
            if extra_column in edge_data:
                extra_values.append(edge_data[extra_column])
                valid_extra_cols.add(extra_column)
        row = tuple([source, itr, target] + extra_values)
        network_array.append(row)

    return pd.DataFrame( network_array, columns=['source', 'interaction', 'target'] + sorted(valid_extra_cols))


def get_node(id):
    node = {
        'data': {
            'id': str(id),
            'name': str(id)
        }
    }
    return node


def get_edge(source, target, interaction, **kwargs):
    if interaction is None:
        itr = '-'
    else:
        itr = interaction

    edge = {
        'data': {
            'source': source,
            'target': target,
            'interaction': itr
        }
    }

    # inserts extra <key:value> in the edge.
    for key, value in kwargs.items():
        edge["data"][key] = value

    return edge



from .base import *

class cyndex2(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/cyndex2'
        self.___url=url


    def updateNetworkInNdex(self, suid, body, verbose=None):
        """
        Update an NDEx network.

        :param suid: Cytoscape Collection/Subnetwork SUID
        :param body: Properties required to update a network record in NDEx.
        :param verbose: print more

        :returns: 200: successful operation; 404: Network does not exist
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/cyndex2/'+sv+'/networks/'+str(suid)+'', method="PUT", body=body, verbose=verbose)
        return response


    def saveNetworkToNdex(self, suid, body, verbose=None):
        """
        Save a network/collection to NDEx

        :param suid: Cytoscape Collection/Subnetwork SUID
        :param body: Properties required to save network to NDEx.
        :param verbose: print more

        :returns: 200: successful operation; 404: Network does not exist
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        PARAMS=set_param(['suid','body'],[suid,body])
        response=api(url=surl+'/cyndex2/'+sv+'/networks/'+str(suid)+'', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def getNetworkSummary(self, suid, verbose=None):
        """
        Returns summary of collection containing the specified network.

        :param suid: Cytoscape Collection/Subnetwork SUID
        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/cyndex2/'+sv+'/networks/'+str(suid)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getAppInfo(self, verbose=None):
        """
        App version and other basic information will be provided.

        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/cyndex2/v1', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateCurrentNetworkInNdex(self, body, verbose=None):
        """
        Update current network's record in NDEx

        :param body: Properties required to update a network record in NDEx.
        :param verbose: print more

        :returns: 200: successful operation; 404: Network does not exist
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/cyndex2/'+sv+'/networks/current', method="PUT", body=body, verbose=verbose)
        return response


    def saveCurrentNetworkToNdex(self, body, verbose=None):
        """
        Save current network/collection to NDEx

        :param body: Properties required to save current network to NDEx.
        :param verbose: print more

        :returns: 200: successful operation; 404: Current network does not exist
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        PARAMS=set_param(['body'],[body])
        response=api(url=surl+'/cyndex2/'+sv+'/networks/current', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def getCurrentNetworkSummary(self, verbose=None):
        """
        Returns summary of collection contains current network.

        :param verbose: print more

        :returns: 200: successful operation
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        response=api(url=surl+'/cyndex2/'+sv+'/networks/current', method="GET", verbose=verbose, parse_params=False)
        return response


    def createNetworkFromNdex(self, body, verbose=None):
        """
        Import network(s) from NDEx.

        :param body: Properties required to import network from NDEx.
        :param verbose: print more

        :returns: 200: successful operation; 404: Network does not exist
        """

        surl=self.___url
        sv=surl.split('/')[-1]
        surl=surl.rstrip(sv+'/')
        PARAMS=set_param(['body'],[body])
        response=api(url=surl+'/cyndex2/'+sv+'/networks', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


from .base import *

class apply(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'layout'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/apply'
        self.___url=url


    def applyLayout(self, algorithmName, networkId, column, verbose=None):
        """
        Applies the Layout specified by the `algorithmName` parameter to the Network specified by the `networkId` parameter. If the Layout is has an option to use a Column, it can be specified by the `column` parameter.

        :param algorithmName: Name of layout algorithm
        :param networkId: SUID of the Network
        :param column: Name of the Column to be used by the Layout -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/layouts/'+str(algorithmName)+'/'+str(networkId)+'', PARAMS={'column':column}, method="GET", verbose=verbose, parse_params=False)
        return response


    def fitContent(self, networkId, verbose=None):
        """
        Fit the first available Network View for the Network specified by the `networkId` parameter to the current window.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/fit/'+str(networkId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getStyleNames(self, verbose=None):
        """
        Returns a list of all Visual Style names. Style names may not be unique.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/styles', method="H", verbose=verbose, parse_params=False)
        return response


    def copyCurrentLayout(self, sourceViewSUID, targetViewSUID, body, verbose=None):
        """
        Copy one network view layout onto another, setting the node location and view scale to match. This makes visually comparing networks simple.

        :param sourceViewSUID: Source network view SUID (or "current")
        :param targetViewSUID: Target network view SUID (or "current")
        :param body: Clone the specified network view layout onto another network view -- Not required, can be None
        :param verbose: print more

        :returns: 200: successful operation; 404: Network View does not exist
        """

        response=api(url=self.___url+'apply/layouts/copycat/'+str(sourceViewSUID)+'/'+str(targetViewSUID)+'', method="PUT", body=body, verbose=verbose)
        return response


    def bundleEdge(self, networkId, verbose=None):
        """
        Apply edge bundling to the Network specified by the `networkId` parameter. Edge bundling is executed with default parameters; at present, optional parameters are not supported.

        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/edgebundling/'+str(networkId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def applyStyle(self, styleName, networkId, verbose=None):
        """
        Applies the Visual Style specified by the `styleName` parameter to the network specified by the `networkId` parameter.

        :param styleName: Name of the Visual Style
        :param networkId: SUID of the Network
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/styles/'+str(styleName)+'/'+str(networkId)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateLayoutParameters(self, algorithmName, body, verbose=None):
        """
        Updates the Layout parameters for the Layout algorithm specified by the `algorithmName` parameter.

        :param algorithmName: Name of the layout algorithm
        :param body: A list of Layout Parameters with Values.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'apply/layouts/'+str(algorithmName)+'/parameters', method="PUT", body=body, verbose=verbose)
        return response


    def getLayoutParameters(self, algorithmName, verbose=None):
        """
        Returns all editable parameters for the Layout algorithm specified by the `algorithmName` parameter.

        :param algorithmName: Name of the Layout algorithm
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/layouts/'+str(algorithmName)+'/parameters', method="H", verbose=verbose, parse_params=False)
        return response
        

    def getLayout(self, algorithmName, verbose=None):
        """
        Returns all the details, including names, parameters, and compatible column types for the Layout algorithm specified by the `algorithmName` parameter.

        :param algorithmName: Name of the Layout algorithm
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/layouts/'+str(algorithmName)+'', method="H", verbose=verbose, parse_params=False)
        return response


    def layoutList(self, verbose=None):
        """
        Returns all available layouts as a list of layout names.
        
        <h3>Important Note</h3>
        
        This <strong>does not include yFiles layout algorithms</strong>, due to license issues.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/layouts', method="H", verbose=verbose, parse_params=False)
        return response


    def getCompatibleColumnDataTypes(self, algorithmName, verbose=None):
        """
        Returns a list of all compatible column data types for the Layout algorithm specified by the `algorithmName` parameter.

        :param algorithmName: Name of layout algorithm
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'apply/layouts/'+str(algorithmName)+'/columntypes', method="H", verbose=verbose, parse_params=False)
        return response




from .base import *

class ui(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/ui'
        self.___url=url


    def updatePanelStatus(self, body, verbose=None):
        """
        Updates the status(es) of available CytoPanels.

        :param body: A list of CytoPanels with states.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'ui/panels', method="PUT", body=body, verbose=verbose)
        return response


    def getAllPanelStatus(self, verbose=None):
        """
        Returns all CytoPanels and their statuses.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'ui/panels', method="GET", verbose=verbose, parse_params=False)
        return response


    def getDesktop(self, verbose=None):
        """
        Returns the status of the Desktop

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'ui', method="GET", verbose=verbose, parse_params=False)
        return response


    def getPanelStatus(self, panelName, verbose=None):
        """
        Returns the status of the CytoPanel specified by the `panelName` parameter.

        :param panelName: Name of the CytoPanel
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'ui/panels/'+str(panelName)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateLodState(self, verbose=None):
        """
        Switch between full graphics details <---> fast rendering mode.
        
        Returns a success message.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'ui/lod', method="PUT", verbose=verbose)
        return response




from .base import *

class view(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'view'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/view'

    def create(self, layout=None, network=None, verbose=False):
        """
        Creates a new view for the passed network and returns the SUID of the
            new view and the original network. If no networks are specified, it
            creates a view for the current network, if there is one.

        :param layout (string, optional): If true (default), the preferred layout
            will be applied to the new view. If false, no layout will be applied.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: eg. 2123

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["layout","network"],[layout,network])
        response=api(url=self.__url+"/create", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def destroy(self, verbose=False):
        """
        Destroys all selected network views and returns their SUIDs. If no views
            are selected, this command does nothing.
        :param verbose: print more

        :returns: eg. [ 2155 ]
        """
        PARAMS={}
        response=api(url=self.__url+"/destroy", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def export(self, Height=None, options=None, outputFile=None, Resolution=None,\
        Units=None, Width=None, Zoom=None, view="current", verbose=False):
        """
        Exports the current view to a graphics file and returns the path to the
            saved file. PNG and JPEG formats have options for scaling, while other
            formats only have the option 'exportTextAsFont'. For the PDF format,
            exporting text as font does not work for two-byte characters such as
            Chinese or Japanese. To avoid corrupted texts in the exported PDF,
            please set false to 'exportTextAsFont' when exporting networks including
            those non-English characters.

        :param Height (string, optional): The height of the exported image. Valid
            only for bitmap formats, such as PNG and JPEG.
        :param options (string, optional): The format of the output file. =
            ['JPEG (*.jpeg, *.jpg)', 'PDF (*.pdf)', 'PNG (*.png)', 'PostScript (*.ps)',
             'SVG (*.svg)']
        :param OutputFile (string, optional): The path name of the file where
            the view must be saved to. By default, the view's title is used as
            the file name.
        :param Resolution (string, optional): The resolution of the exported
            image, in DPI. Valid only for bitmap formats, when the selected width
            and height 'units' is inches. The possible values are: 72 (default),
            100, 150, 300, 600. = ['72', '100', '150', '300', '600']
        :param Units (string, optional): The units for the 'width' and 'height'
            values. Valid only for bitmap formats, such as PNG and JPEG. The
            possible values are: pixels (default), inches. = ['pixels', 'inches']
        :param Width (string, optional): The width of the exported image. Valid
            only for bitmap formats, such as PNG and JPEG.
        :param Zoom (string, optional): The zoom value to proportionally scale
            the image. The default value is 100.0. Valid only for bitmap formats,
            such as PNG and JPEG
        :param verbose: print more

        :returns: path to the saved file
        """
        PARAMS=set_param(["Height","options","outputFile","Resolution",\
        "Units","Width","Zoom","view"],\
        [Height,options,outputFile,Resolution,Units,Width,Zoom,view ])
        response=api(url=self.__url+"/export", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def fit_content(self, verbose=False):
        """
        Zooms out the current view in order to display all of its elements.

        :param verbose: print more

        """
        PARAMS={}
        response=api(url=self.__url+"/fit content", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response 

    def fit_selected(self, verbose=False):
        """
        Changes the current view's zoom and viewport so the selected nodes and
            edges fit into the view area.

        :param verbose: print more

        """
        PARAMS={}
        response=api(url=self.__url+"/fit selected", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response
    
    def get_current(self, layout=None, network=None, verbose=False):
        """
        Returns the current view or null if there is none.

        :param verbose: print more

        :returns: current view or null if there is none
        """
        PARAMS={}
        response=api(url=self.__url+"/get_current", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def list(self, network=None, verbose=False):
        """
        Returns a list with the passed network's views or an empty list if
            there are no views. If a network is not specified, it assumes the
            current network.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        :returns: a list with the passed network's views or an empty list if there are no views

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/list", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def set_current(self, network=None, verbose=False):
        """
        Sets the current view, which can also be null.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/set current", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def update(self, network=None, verbose=False):
        """
        Sets the current view, which can also be null.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/update", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response



# -*- coding: utf-8 -*-
"""

Conversion utilities for igraph

"""

try:
    import igraph as ig
except Exception as error:
    print(str(error))
    print("py2cytoscape: Error importing igraph. You won't be able to import from igraph.")
    ig = None

DEF_SCALING = 100.0


def from_igraph(igraph_network, layout=None, scale=DEF_SCALING):
    if ig is None:
        raise ImportError('igraph not found')

    new_graph = {}
    network_data = {}
    elements = {}
    nodes = []
    edges = []

    # Convert network attributes
    network_attr = igraph_network.attributes()
    for key in network_attr:
        network_data[key] = igraph_network[key]

    # get network as a list of edges
    edges_original = igraph_network.es
    nodes_original = igraph_network.vs

    node_attr = igraph_network.vs.attributes()
    for idx, node in enumerate(nodes_original):
        new_node = {}
        data = {}
        data['id'] = str(node.index)
        data['name'] = str(node.index)
        for key in node_attr:
            data[key] = node[key]
        new_node['data'] = data
        if layout is not None:
            position = {}
            position['x'] = layout[idx][0] * scale
            position['y'] = layout[idx][1] * scale
            new_node['position'] = position

        nodes.append(new_node)

    # Add edges to the elements
    edge_attr = igraph_network.es.attributes()
    for edge in edges_original:
        new_edge = {}
        data = {}
        data['source'] = str(edge.source)
        data['target'] = str(edge.target)
        for key in edge_attr:
            data[key] = edge[key]
        new_edge['data'] = data
        edges.append(new_edge)

    elements['nodes'] = nodes
    elements['edges'] = edges
    new_graph['elements'] = elements
    new_graph['data'] = network_data

    return new_graph


def to_igraph(network):

    nodes = network['elements']['nodes']
    edges = network['elements']['edges']
    network_attr = network['data']

    node_count = len(nodes)
    edge_count = len(edges)

    g = ig.Graph()

    # Graph attributes
    for key in network_attr.keys():
        g[key] = network_attr[key]

    g.add_vertices(nodes)

    # Add node attributes
    node_attributes = {}
    node_id_dict = {}
    for i, node in enumerate(nodes):
        data = node['data']
        for key in data.keys():
            if key not in node_attributes:
                node_attributes[key] = [None] * node_count

            # Save index to map
            if key == 'id':
                node_id_dict[data[key]] = i

            node_attributes[key][i] = data[key]

    for key in node_attributes.keys():
        g.vs[key] = node_attributes[key]

    # Create edges
    edge_tuples = []
    edge_attributes = {}
    for i, edge in enumerate(edges):
        data = edge['data']
        source = data['source']
        target = data['target']
        edge_tuple = (node_id_dict[source], node_id_dict[target])
        edge_tuples.append(edge_tuple)
        for key in data.keys():
            if key not in edge_attributes:
                edge_attributes[key] = [None] * edge_count

            # Save index to map
            edge_attributes[key][i] = data[key]

    g.add_edges(edge_tuples)

    # Assign edge attributes
    for key in edge_attributes.keys():
        if key == 'source' or key == 'target':
            continue
        else:
            g.es[key] = edge_attributes[key]

    return g



import requests

HEADERS = {'Content-Type': 'application/json'}


class LayoutClient(object):
    def __init__(self, url):
        self.__url = url + 'apply/layouts'
        self.__base_url = url

    def get_all(self):
        return requests.get(self.__url).json()

    def apply(self, name='force-directed', network=None):
        if network is None:
            raise ValueError('Target network is required')

        url = self.__url + '/' + name + '/' + str(network.get_id())
        requests.get(url)

    def bundle_edge(self, network=None):
        if network is None:
            raise ValueError('Target network is required')

        url = self.__base_url + 'apply/edgebundling/' + str(network.get_id())
        requests.get(url)

    def fit(self, network=None):
        if network is None:
            raise ValueError('Target network is required')
        url = self.__base_url + 'apply/fit/' + str(network.get_id())
        requests.get(url)

    def apply_from_presets(self, network=None, positions=None):
        if network is None:
            raise ValueError('Target network is required')

        views = network.get_views()
        view_id = views[0]

        manual_layout_url = self.__base_url +  'networks/' \
                            + str(network.get_id()) \
                            + '/views/' \
                            + str(view_id) + '/nodes'
        pos = self.__apply_preset(locations=positions)

        return requests.put(manual_layout_url, json=pos, headers=HEADERS)

    def __apply_preset(self, locations):
        position_list = []
        for location in locations:
            position = {
                'SUID': location[0],
                'view': [
                    {
                        'visualProperty': 'NODE_X_LOCATION',
                        'value': location[1]
                    },
                    {
                        'visualProperty': 'NODE_Y_LOCATION',
                        'value': location[2]
                    }
                ]
            }
            position_list.append(position)
        return position_list


class EdgeBundlingClient(object):
    def __init__(self, url):
        self.__url = url + 'apply/edgebundling'

    def apply(self, network=None):
        if network is None:
            raise ValueError('Target network is required')

        url = self.__url + '/' + str(network.get_id())
        requests.get(url)



from .base import *

class styles(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/styles'
        self.___url= url


    def updateDefaults(self, name, body, verbose=None):
        """
        Updates the default values for the Visual Properties in the Visual Style specified by the `name` parameter.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style
        :param body: A list of Visual Property values to update.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/defaults', method="PUT", body=body, verbose=verbose)
        return response


    def getDefaults(self, name, verbose=None):
        """
        Returns a list of all the default values for the Visual Style specified by the `name` parameter.

        :param name: Name of the Visual Style
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/defaults', method="GET", verbose=verbose, parse_params=False)
        return response


    def getSingleVisualProperty(self, visualProperty, verbose=None):
        """
        Return the Visual Property specified by the `visualProperty` parameter.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param visualProperty: ID of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/visualproperties/'+str(visualProperty)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def createStyle(self, body, verbose=None):
        """
        Creates a new Visual Style using the message body.
        
        Returns the title of the new Visual Style. If the title of the Visual Style already existed in the session, a new one will be automatically generated and returned.

        :param body: The details of the new Visual Style to be created.
        :param verbose: print more

        :returns: 200: successful operation
        """

        PARAMS=set_param(['body'],[body])
        response=api(url=self.___url+'styles', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def deleteAllStyles(self, verbose=None):
        """
        Deletes all vision styles except for default style

        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles', method="DELETE", verbose=verbose)
        return response


    def getStyleNames(self, verbose=None):
        """
        Returns a list of all the Visual Style names in the current session.

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateDependencies(self, name, body, verbose=None):
        """
        Sets the value of Visual Property dependencies to the values in the message body.

        :param name: Name of the Visual Style
        :param body: A list of dependencies.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/dependencies', method="PUT", body=body, verbose=verbose)
        return response


    def getAllDependencies(self, name, verbose=None):
        """
        Returns the status of all the Visual Property Dependencies.

        :param name: Name of the Visual Style
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/dependencies', method="GET", verbose=verbose, parse_params=False)
        return response


    def getStyle(self, name, verbose=None):
        """
        Visual Style in [Cytoscape.js CSS](http://js.cytoscape.org/#style) format.

        :param name: Name of the Visual Style
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'.json', method="GET", verbose=verbose, parse_params=False)
        return response


    def getStyleCount(self, verbose=None):
        """
        Returns the number of Visual Styles available in the current session

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/count', method="GET", verbose=verbose, parse_params=False)
        return response


    def deleteMapping(self, name, vpName, verbose=None):
        """
        Deletes the Visual Property mapping specified by the `vpName` and `name` parameters.

        :param name: Name of the Visual Style containing the Visual Mapping
        :param vpName: Name of the Visual Property that the Visual Mapping controls
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/mappings/'+str(vpName)+'', method="DELETE", verbose=verbose)
        return response


    def getRangeValues(self, vp, verbose=None):
        """
        Returns a list of all available values for the Visual Property specified by the `vp` parameter.
        
        This method is only for Visual Properties with a Discrete Range, such as NODE_SHAPE or EDGE_LINE_TYPE.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param vp: ID of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/visualproperties/'+str(vp)+'/values', method="GET", verbose=verbose, parse_params=False)
        return response


    def updateStyleName(self, name, body, verbose=None):
        """
        Updates the name of the Visual Style specified by the `name` parameter.

        :param name: Original name of the Visual Style
        :param body: A new name for the Visual Style.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'', method="PUT", body=body, verbose=verbose)
        return response


    def deleteStyle(self, name, verbose=None):
        """
        Deletes the Visual Style specified by the `name` parameter.

        :param name: Visual Style Name
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'', method="DELETE", verbose=verbose)
        return response


    def getStyleFull(self, name, verbose=None):
        """
        Returns the Visual Style specified by the `name` parameter.

        :param name: Name of the Visual Style
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def createMappings(self, name, body, verbose=None):
        """
        Create a new Visual Mapping function and add it to the Visual Style specified by the `name` parameter. Existing mappings in the Visual Style will be overidden by the new mappings created.
        
        The types of mapping available in Cytoscape are explained in depth [here](http://manual.cytoscape.org/en/stable/Styles.html#how-mappings-work). An example of the data format for each is included below. For additional details, such as what Visual Properties supported by each Mapping, click on the relevant JavaDoc API link.
        
        #### Discrete Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/DiscreteMapping.html)
        ```
        { "mappingType": "discrete",
        "mappingColumn": "interaction",
            "mappingColumnType": "String",
            "visualProperty": "EDGE_WIDTH", 
            "map": [
            { "key" : "pd",
                "value" : "20"
            },
            { "key" : "pp",
                "value" : "1.5"
            }
            ]
        }```
        #### Continuous Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/ContinuousMapping.html)
        ```
        { "mappingType": "continuous",
        "mappingColumn": "Degree",
        "mappingColumnType": "Integer",
        "visualProperty": "NODE_SIZE",
        "points": [
            { "value" : 1,
                "lesser" : "20",
                "equal" : "20",
                "greater" : "20"
            },
                { "value" : 20,
                "lesser" : "120",
                "equal" : "120",
                "greater" : "220"      }
            ]
        }```
        #### Passthrough Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/PassthroughMapping.html)
        ```
        { "mappingType": "passthrough",
        "mappingColumn": "name",
        "mappingColumnType": "String",
        "visualProperty": "EDGE_LABEL" 
        }```
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style
        :param body: A list of new mappings.
        :param verbose: print more

        :returns: default: successful operation
        """

        PARAMS=set_param(['name','body'],[name,body])
        response=api(url=self.___url+'styles/'+str(name)+'/mappings', PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def getMappings(self, name, verbose=None):
        """
        Returns a list of all Visual Mappings used in the Visual Style specified by the `name` parameter.
        
        The types of mapping available in Cytoscape are explained in depth [here](http://manual.cytoscape.org/en/stable/Styles.html#how-mappings-work). An example of the data format for each is included below. For additional details, such as what Visual Properties supported by each Mapping, click on the relevant JavaDoc API link.
        
        #### Discrete Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/DiscreteMapping.html)
        ```
        { "mappingType": "discrete",
        "mappingColumn": "interaction",
            "mappingColumnType": "String",
            "visualProperty": "EDGE_WIDTH", 
            "map": [
            { "key" : "pd",
                "value" : "20"
            },
            { "key" : "pp",
                "value" : "1.5"
            }
            ]
        }```
        #### Continuous Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/ContinuousMapping.html)
        ```
        { "mappingType": "continuous",
        "mappingColumn": "Degree",
        "mappingColumnType": "Integer",
        "visualProperty": "NODE_SIZE",
        "points": [
            { "value" : 1,
                "lesser" : "20",
                "equal" : "20",
                "greater" : "20"
            },
                { "value" : 20,
                "lesser" : "120",
                "equal" : "120",
                "greater" : "220"      }
            ]
        }```
        #### Passthrough Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/PassthroughMapping.html)
        ```
        { "mappingType": "passthrough",
        "mappingColumn": "name",
        "mappingColumnType": "String",
        "visualProperty": "EDGE_LABEL" 
        }```
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/mappings', method="GET", verbose=verbose, parse_params=False)
        return response


    def getVisualProperties(self, verbose=None):
        """
        Get all available Visual Properties.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/visualproperties', method="H", verbose=verbose, parse_params=False)
        return response


    def updateMapping(self, name, vp, body, verbose=None):
        """
        Update the visual mapping specified by the `name` and `vp` parameters.
        
        The types of mapping available in Cytoscape are explained in depth [here](http://manual.cytoscape.org/en/stable/Styles.html#how-mappings-work). An example of the data format for each is included below. For additional details, such as what Visual Properties supported by each Mapping, click on the relevant JavaDoc API link.
        
        #### Discrete Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/DiscreteMapping.html)
        ```
        { "mappingType": "discrete",
        "mappingColumn": "interaction",
            "mappingColumnType": "String",
            "visualProperty": "EDGE_WIDTH", 
            "map": [
            { "key" : "pd",
                "value" : "20"
            },
            { "key" : "pp",
                "value" : "1.5"
            }
            ]
        }```
        #### Continuous Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/ContinuousMapping.html)
        ```
        { "mappingType": "continuous",
        "mappingColumn": "Degree",
        "mappingColumnType": "Integer",
        "visualProperty": "NODE_SIZE",
        "points": [
            { "value" : 1,
                "lesser" : "20",
                "equal" : "20",
                "greater" : "20"
            },
                { "value" : 20,
                "lesser" : "120",
                "equal" : "120",
                "greater" : "220"      }
            ]
        }```
        #### Passthrough Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/PassthroughMapping.html)
        ```
        { "mappingType": "passthrough",
        "mappingColumn": "name",
        "mappingColumnType": "String",
        "visualProperty": "EDGE_LABEL" 
        }```
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style containing the Visual Mapping
        :param vp: Name of the Visual Property that the Visual Mapping controls
        :param body: A list of new mappings.
        :param verbose: print more

        :returns: default: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/mappings/'+str(vp)+'', method="PUT", body=body, verbose=verbose)
        return response


    def getMapping(self, name, vp, verbose=None):
        """
        Returns the Visual Mapping assigned to the Visual Property specified by the `name` and `vp` parameters.
        
        The types of mapping available in Cytoscape are explained in depth [here](http://manual.cytoscape.org/en/stable/Styles.html#how-mappings-work). An example of the data format for each is included below. For additional details, such as what Visual Properties supported by each Mapping, click on the relevant JavaDoc API link.
        
        #### Discrete Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/DiscreteMapping.html)
        ```
        { "mappingType": "discrete",
        "mappingColumn": "interaction",
            "mappingColumnType": "String",
            "visualProperty": "EDGE_WIDTH", 
            "map": [
            { "key" : "pd",
                "value" : "20"
            },
            { "key" : "pp",
                "value" : "1.5"
            }
            ]
        }```
        #### Continuous Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/ContinuousMapping.html)
        ```
        { "mappingType": "continuous",
        "mappingColumn": "Degree",
        "mappingColumnType": "Integer",
        "visualProperty": "NODE_SIZE",
        "points": [
            { "value" : 1,
                "lesser" : "20",
                "equal" : "20",
                "greater" : "20"
            },
                { "value" : 20,
                "lesser" : "120",
                "equal" : "120",
                "greater" : "220"      }
            ]
        }```
        #### Passthrough Mapping
        [JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/vizmap/mappings/PassthroughMapping.html)
        ```
        { "mappingType": "passthrough",
        "mappingColumn": "name",
        "mappingColumnType": "String",
        "visualProperty": "EDGE_LABEL" 
        }```
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style containing the Visual Property mapping
        :param vp: Name of the Visual Property that the Visual Mapping controls
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/mappings/'+str(vp)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    def getDefaultValue(self, name, vp, verbose=None):
        """
        Returns the default value for the Visual Property specified by the `name` and `vp` parameters.
        
        Additional details on common Visual Properties can be found in the [Basic Visual Lexicon JavaDoc API](http://chianti.ucsd.edu/cytoscape-3.6.1/API/org/cytoscape/view/presentation/property/BasicVisualLexicon.html)

        :param name: Name of the Visual Style containing the Visual Property
        :param vp: Name of the Visual Property
        :param verbose: print more

        :returns: 200: successful operation
        """

        response=api(url=self.___url+'styles/'+str(name)+'/defaults/'+str(vp)+'', method="GET", verbose=verbose, parse_params=False)
        return response


    


from .base import *

class group(object):
    """
    cytoscape session interface as shown in CyREST's swagger documentation for 'group'.

    :param url: an url of the type 'http://' + host + ':' + str(port) + '/' + version + '/'.
    """

    def __init__(self, url):
        self.__url = url + 'commands/group'

    def add(self, edgeList=None, groupName=None, network=None, nodeList=None, verbose=False):
        """
        Adds the specified nodes and edges to the specified group.

        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param groupName (string, optional): Specifies the name used to
            identify the group.
        :param network (string, optional): Specifies a network by name, or
            by SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","groupName","network","nodeList"],[edgeList,groupName,network,nodeList])
        response=api(url=self.__url+"/add", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def collapse(self, groupList=None, network=None, verbose=False):
        """
        Replaces the representation of all of the nodes and edges in a group with a single node.

        :param groupList (string, optional): Specifies a list of groups. The
            keywords all, selected, or unselected can be used to specify groups
            by their selection state.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["groupList","network"],[groupList,network])
        response=api(url=self.__url+"/collapse", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def create(self, groupName=None, network=None, nodeList=None, verbose=False):
        """
        Create a group from the specified nodes.

        :param groupName (string, optional): Specifies the name used to identify
            the group.
        :param network (string, optional): Specifies a network by name, or
            by SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more

        :return: SUID of created group

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["groupName","network","nodeList"],[groupName,network,nodeList])
        response=api(url=self.__url+"/create", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def expand(self, groupList=None, network=None, verbose=False):
        """
        Replaces the group node with member nodes for a set of groups.

        :param groupList (string, optional): Specifies a list of groups. The
            keywords all, selected, or unselected can be used to specify groups
            by their selection state.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param verbose: print more

        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["groupList","network"],[groupList,network])
        response=api(url=self.__url+"/expand", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def get(self, network=None, node=None, verbose=False):
        """
        Get a group by providing a network and the group node identifier.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param node (string, optional): Selects a node by name, or, if the
            parameter has the prefix suid:, selects a node by SUID.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","node"],[network,node])
        response=api(url=self.__url+"/get", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def glist(self, network=None, verbose=False):
        """
        Lists the SUIDs of all of the groups in a network.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be to specify the current network.
        :param verbose: print more

        :returns: eg. [ 3545 ]
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network"],[network])
        response=api(url=self.__url+"/list", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def remove(self, edgeList=None, groupName=None, network=None, nodeList=None, verbose=False):
        """
        Remove the selected nodes and edges from their current group.

        :param edgeList (string, optional): Specifies a list of edges. The
            keywords all, selected, or unselected can be used to specify edges
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param groupName (string, optional): Specifies the name used to identify
            the group.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list
            of COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["edgeList","groupName","network","nodeList"],[edgeList,groupName,network,nodeList])
        response=api(url=self.__url+"/remove", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response

    def rename(self, groupName=None, network=None, newName=None, verbose=False):
        """
        Changes the name of the selected group or groups.

        :param groupName (string, optional): Specifies the name used to identify
            the group.
        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param newName (string, optional): Specifies the NEW name used to
            identify the group.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","nodeList","newName"],[network,nodeList,newName])
        response=api(url=self.__url+"/rename", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response


    def ungroup(self, network=None, nodeList=None, verbose=False):
        """
        Ungroups one or more groups, expanding them if they are collapsed and removing the group nodes.

        :param network (string, optional): Specifies a network by name, or by
            SUID if the prefix SUID: is used. The keyword CURRENT, or a blank
            value can also be used to specify the current network.
        :param nodeList (string, optional): Specifies a list of nodes. The
            keywords all, selected, or unselected can be used to specify nodes
            by their selection state. The pattern COLUMN:VALUE sets this parameter
            to any rows that contain the specified column value; if the COLUMN
            prefix is not used, the NAME column is matched by default. A list of
            COLUMN:VALUE pairs of the format COLUMN1:VALUE1,COLUMN2:VALUE2,...
            can be used to match multiple values.
        :param verbose: print more
        """
        network=check_network(self,network,verbose=verbose)
        PARAMS=set_param(["network","nodeList"],[network,nodeList])
        response=api(url=self.__url+"/ungroup", PARAMS=PARAMS, method="POST", verbose=verbose)
        return response
